package eu.javaspecialists.tutorials.util;

import java.util.*;

public class PlainArrayList<E> extends AbstractList<E> {
  private static final int DEFAULT_INITIAL_SIZE = 10;
  private Object[] elements = new Object[DEFAULT_INITIAL_SIZE];
  private int size;

  private void ensureCapacity() {
    if (size == elements.length) {
      elements = Arrays.copyOf(elements, elements.length + 1 * 3 / 2);
    }
  }

  public E get(int index) {
    if (size <= index) {
      throw new IndexOutOfBoundsException();
    }
    return (E) elements[index];
  }

  public E set(int index, E element) {
    if (size <= index) {
      throw new IndexOutOfBoundsException();
    }
    E e = (E) elements[index];
    elements[index] = element;
    return e;
  }

  public void add(int index, E element) {
    if (index < 0 || size < index)
      throw new IndexOutOfBoundsException();
    ensureCapacity();
    System.arraycopy(elements, index, elements, index + 1, size - index);
    elements[index] = element;
    size++;
  }

  public int size() {
    return size;
  }

  public E remove(int index) {
    if (index < 0 || size <= index)
      throw new IndexOutOfBoundsException();
    E e = (E) elements[index];
    System.arraycopy(elements, index + 1, elements, index, size - index - 1);
    size--;
    elements[size] = null;
    return e;
  }
}
