package eu.javaspecialists.tutorials.util;

import org.apache.commons.collections4.list.*;

import java.util.*;

public abstract class AbstractCircularArrayListTest<E> extends AbstractListTest<E> {
  public AbstractCircularArrayListTest(String testName) {
    super(testName);
  }

  public void testLaBranche() {
    List<Object> cal = (List)makeObject();
    List<Object> al = new ArrayList<>();

    assertEquals(0, cal.size());
    cal.add(cal.size(), "Dog");
    al.add(al.size(), "Dog");
    assertEquals(al, cal);

    cal.add(cal.size(), "Cat");
    al.add(al.size(), "Cat");
    assertEquals(al, cal);

    List<Object> innerCal = (List<Object>)makeObject();
    List<Object> innerAl = new ArrayList<>();
    innerCal.add(innerCal.size(), "FoxFire");
    innerAl.add(innerAl.size(), "FoxFire");
    assertEquals(innerAl, innerCal);

    cal.add(cal.size(), innerCal);
    al.add(al.size(), innerAl);
    assertEquals(al, cal);

    cal.add(1, "Dragon");
    al.add(1, "Dragon");
    assertEquals(al, cal);

    cal.add(0, "Race Car");
    al.add(0, "Race Car");
    assertEquals(al, cal);

    System.out.println("Object: " + cal.toString());
  }

  public void testMultipleRemoveFromIterator() {
    List<E> list = makeFullCollection();
    Iterator<E> it = list.iterator();
    try {
      it.remove();
    } catch (IllegalStateException success) {
    }
    it.next();
    it.remove();
    try {
      it.remove();
    } catch (IllegalStateException success) {
    }
  }

  public void testRemoveAndAddingInMiddle() {
    List<E> list = makeFullCollection();
    System.out.println(list);
    List<E> copy = new ArrayList<>(list);
    List<E> removed = new ArrayList<>();
    for (int i = 0; i < 3; i++) {
      E removeFrontList = list.remove(0);
      E removeEndList = list.remove(list.size() - 1);
      E removeMiddleList = list.remove((int) (list.size() / 2));
      E removeFrontCopy = copy.remove(0);
      E removeEndCopy = copy.remove(copy.size() - 1);
      E removeMiddleCopy = copy.remove((int) (copy.size() / 2));
      assertEquals(removeFrontList, removeFrontCopy);
      assertEquals(removeEndList, removeEndCopy);
      assertEquals(removeMiddleList, removeMiddleCopy);
      removed.add(removeFrontList);
      removed.add(removeMiddleList);
      removed.add(removeEndList);
    }
    assertEquals(list, copy);

    System.out.println("removed = " + removed);

    for (int i = 0; i < 3; i++) {
      E elementToAddFirst = removed.remove(0);
      E elementToAddEnd = removed.remove(0);
      E elementToAddMiddle = removed.remove(0);
      list.add(list.size() / 2, elementToAddMiddle);
      list.add(0, elementToAddFirst);
      list.add(0, elementToAddFirst);
      list.add(0, elementToAddFirst);
      list.add(list.size() - 1, elementToAddEnd);
      copy.add(copy.size() / 2, elementToAddMiddle);
      copy.add(0, elementToAddFirst);
      copy.add(0, elementToAddFirst);
      copy.add(0, elementToAddFirst);
      copy.add(copy.size() - 1, elementToAddEnd);
    }
    assertEquals(list, copy);
    System.out.println("list = " + list);
    System.out.println("copy = " + copy);
  }

  public void testConcurrentModificationException() {
    List<E> list = makeFullCollection();

    Iterator<E> iterator;

    iterator = list.iterator();
    list.add(list.get(0));
    try {
      iterator.next();
      fail("Expected ConcurrentModificationException after an add(e)");
    } catch (ConcurrentModificationException success) { }

    iterator = list.iterator();
    list.add(0, list.get(0));
    try {
      iterator.next();
      fail("Expected ConcurrentModificationException after an add(int, e)");
    } catch (ConcurrentModificationException success) { }

    iterator = list.iterator();
    list.remove(0);
    try {
      iterator.next();
      fail("Expected ConcurrentModificationException after a remove(int)");
    } catch (ConcurrentModificationException success) { }

    iterator = list.iterator();
    list.set(0, list.get(2));
    iterator.next(); // should not cause a ConcurrentModificationException
  }

  public final boolean isTestSerialization() {
    return false;
  }
}
